//
//  URLLoader.m
//  Mensa
//
//  Created by Jonathan Diehl on 20.12.10.
//  Copyright 2010 RWTH. All rights reserved.
//

#import "URLLoader.h"


@implementation URLLoader

@synthesize delegate, request;


// designated init
- (id)initWithRequest:(NSURLRequest *)theRequest;
{
	self = [super init];
	if (self != nil) {
		[self retain];
		request = [theRequest retain];
	}
	return self;
}

// create url request from url
- (id)initWithURL:(NSURL *)url;
{
	NSURLRequest *aRequest = [NSURLRequest requestWithURL:url];
	return [self initWithRequest:aRequest];
}

// start loading
- (void)start;
{
	// already started?
	if(connection) return;
	
	// initialize mutable data
	data = [[NSMutableData alloc] init];
	
	// initialize and start connection
	connection = [[NSURLConnection alloc] initWithRequest:request delegate:self];
	[connection start];
}

// cancel loading
- (void)cancel;
{
	[connection cancel];
	[connection release];
	connection = nil;
	
	[data release];
	data = nil;
}

// cleanup
- (void)dealloc
{
	[self cancel];
	[request release];
	[super dealloc];
}


#pragma mark NSURLConnection delegate

// received data
- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)someData
{
	[data appendData:someData];
}

// finished
- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
	// inform delegate
	[self.delegate urlLoader:self didLoadData:[NSData dataWithData:data]];
	
	// clean up
	[self cancel];
}

// failed
- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
	[self.delegate urlLoader:self didFailWithError:error];
	
	// clean up
	[self cancel];
}


@end
